@@ -1,22 +1,36 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 """this module contains a set of functions to handle inference on astng trees
 
-:author:    Sylvain Thenault
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2009 Sylvain Thenault
-:contact:   mailto:thenault@gmail.com
 """
 
 from __future__ import generators
@@ -30,10 +44,10 @@ try:
 except NameError:
     class GeneratorExit(Exception): pass
 
-from logilab.astng import MANAGER, _nodes as nodes, raw_building
+from logilab.astng import MANAGER, nodes, raw_building
 from logilab.astng import ASTNGError, InferenceError, UnresolvableName, \
      NoDefault, NotFoundError, ASTNGBuildingException
-from logilab.astng.infutils import YES, Instance, InferenceContext, \
+from logilab.astng.bases import YES, Instance, InferenceContext, \
      _infer_stmts, copy_context, path_wrapper, raise_if_nothing_infered
 from logilab.astng.protocols import _arguments_infer_argname
 
@@ -49,29 +63,26 @@ _CONST_PROXY = {
     }
 _CONST_PROXY[type(None)].parent = _CONST_PROXY[bool].parent
 
+# TODO : find a nicer way to handle this situation; we should at least
+# be able to avoid calling MANAGER.astng_from_class(const.value.__class__)
+# each time (if we can not avoid the property). However __proxied introduced an
+# infinite recursion (see https://bugs.launchpad.net/pylint/+bug/456870)
 def _set_proxied(const):
-    if not hasattr(const, '__proxied'):
-        const.__proxied = _CONST_PROXY[const.value.__class__]
-    return const.__proxied
+    return _CONST_PROXY[const.value.__class__]
 nodes.Const._proxied = property(_set_proxied)
 
-nodes.Const.__bases__ += (Instance,)
-
 def Const_pytype(self):
     return self._proxied.qname()
 nodes.Const.pytype = Const_pytype
 
 
 nodes.List._proxied = MANAGER.astng_from_class(list)
-nodes.List.__bases__ += (Instance,)
 nodes.Tuple._proxied = MANAGER.astng_from_class(tuple)
-nodes.Tuple.__bases__ += (Instance,)
-nodes.Dict.__bases__ += (Instance,)
 nodes.Dict._proxied = MANAGER.astng_from_class(dict)
 
 
 class CallContext:
-    """when infering a function call, this class is used to remember values
+    """when inferring a function call, this class is used to remember values
     given as argument
     """
     def __init__(self, args, starargs, dstarargs):
@@ -86,7 +97,7 @@ class CallContext:
         self.dstarargs = dstarargs
 
     def infer_argument(self, funcnode, name, context):
-        """infer a function argument value according the the call context"""
+        """infer a function argument value according to the call context"""
         # 1. search in named keywords
         try:
             return self.nargs[name].infer(context)
@@ -157,14 +168,6 @@ class CallContext:
 
 # .infer method ###############################################################
 
-# extend all classes instead of base Node class which is an unextendable type
-# in 2.6
-def infer_default(self, context=None):
-    """we don't know how to resolve a statement by default"""
-    raise InferenceError(self.__class__.__name__)
-for cls in nodes.ALL_NODES:
-    cls.infer = infer_default
-
 
 def infer_end(self, context=None):
     """inference's end for node such as Module, Class, Function, Const...
@@ -187,7 +190,6 @@ def infer_name(self, context=None):
         raise UnresolvableName(self.name)
     context = context.clone()
     context.lookupname = self.name
-    #context.boundnode = None
     return _infer_stmts(stmts, context, frame)
 nodes.Name.infer = path_wrapper(infer_name)
 nodes.AssName.infer_lhs = infer_name # won't work with a path wrapper
@@ -224,7 +226,7 @@ def infer_import(self, context=None, asname=True):
 nodes.Import.infer = path_wrapper(infer_import)
 
 def infer_name_module(self, name):
-    context = InferenceContext(self)
+    context = InferenceContext()
     context.lookupname = name
     return self.infer(context, asname=False)
 nodes.Import.infer_name_module = infer_name_module
@@ -298,12 +300,14 @@ def infer_subscript(self, context=None):
     else:
         raise InferenceError()
 nodes.Subscript.infer = path_wrapper(infer_subscript)
+nodes.Subscript.infer_lhs = raise_if_nothing_infered(infer_subscript)
 
 
-UNARY_OP_METHOD = {'+':  '__pos__',
-                 '-':  '__neg__',
-                 'not': None, # XXX not '__nonzero__'
-                 }
+UNARY_OP_METHOD = {'+': '__pos__',
+                   '-': '__neg__',
+                   '~': '__invert__',
+                   'not': None, # XXX not '__nonzero__'
+                  }
 
 def infer_unaryop(self, context=None):
     for operand in self.operand.infer(context):
@@ -419,3 +423,4 @@ def infer_empty_node(self, context=None):
         except ASTNGError:
             yield YES
 nodes.EmptyNode.infer = path_wrapper(infer_empty_node)
+
