@@ -1,24 +1,38 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""astng manager: avoid multible astng build of a same module when
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
+"""astng manager: avoid multiple astng build of a same module when
 possible by providing a class responsible to get astng representation
 from various source and using a cache of built modules)
 
-:author:    Sylvain Thenault
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2009 Sylvain Thenault
-:contact:   mailto:thenault@gmail.com
 """
 
 __docformat__ = "restructuredtext en"
@@ -28,22 +42,22 @@ import os
 from os.path import dirname, basename, abspath, join, isdir, exists
 
 from logilab.common.modutils import NoSourceFile, is_python_source, \
-     file_from_modpath, load_module_from_name, \
+     file_from_modpath, load_module_from_name, modpath_from_file, \
      get_module_files, get_source_file, zipimport
 from logilab.common.configuration import OptionsProviderMixIn
 
-from logilab.astng import ASTNGBuildingException, _nodes as nodes, infutils
+from logilab.astng._exceptions import ASTNGBuildingException
 
 def astng_wrapper(func, modname):
     """wrapper to give to ASTNGManager.project_from_files"""
     print 'parsing %s...' % modname
     try:
         return func(modname)
-    except ASTNGBuildingException, ex:
-        print ex
+    except ASTNGBuildingException, exc:
+        print exc
     except KeyboardInterrupt:
         raise
-    except Exception, ex:
+    except Exception, exc:
         import traceback
         traceback.print_exc()
 
@@ -94,13 +108,12 @@ class ASTNGManager(OptionsProviderMixIn):
             self.__dict__ = ASTNGManager.brain
         if not self.__dict__:
             OptionsProviderMixIn.__init__(self)
-            self._cache = None
-            self._mod_file_cache = None
-            self.set_cache_size(200)
             self.load_defaults()
+            # NOTE: cache entries are added by the [re]builder
+            self._cache = {} #Cache(cache_size)
+            self._mod_file_cache = {}
 
-    def set_cache_size(self, cache_size):
-        """set the cache size (flush it as a side effect!)"""
+    def reset_cache(self):
         self._cache = {} #Cache(cache_size)
         self._mod_file_cache = {}
 
@@ -117,56 +130,53 @@ class ASTNGManager(OptionsProviderMixIn):
             source = True
         except NoSourceFile:
             source = False
-        try:
-            return self._cache[filepath]
-        except KeyError:
-            if source:
-                try:
-                    from logilab.astng.builder import ASTNGBuilder
-                    astng = ASTNGBuilder(self).file_build(filepath, modname)
-                except (SyntaxError, KeyboardInterrupt, SystemExit):
-                    raise
-                except Exception, ex:
-                    if __debug__:
-                        print 'error while building astng for', filepath
-                        import traceback
-                        traceback.print_exc()
-                    msg = 'Unable to load module %s (%s)' % (modname, ex)
-                    raise ASTNGBuildingException(msg), None, sys.exc_info()[-1]
-            elif fallback and modname:
-                return self.astng_from_module_name(modname)
-            else:
-                raise ASTNGBuildingException('unable to get astng for file %s' %
-                                             filepath)
-        self._cache[filepath] = astng
-        return astng
-
-    from_file = astng_from_file # backward compat
+        if modname is None:
+            modname = '.'.join(modpath_from_file(filepath))
+        if modname in self._cache:
+            return self._cache[modname]
+        if source:
+            try:
+                from logilab.astng.builder import ASTNGBuilder
+                return ASTNGBuilder(self).file_build(filepath, modname)
+            except (SyntaxError, KeyboardInterrupt, SystemExit):
+                raise
+            except Exception, ex:
+                raise
+                if __debug__:
+                    print 'error while building astng for', filepath
+                    import traceback
+                    traceback.print_exc()
+                msg = 'Unable to load module %s (%s)' % (modname, ex)
+                raise ASTNGBuildingException(msg), None, sys.exc_info()[-1]
+        elif fallback and modname:
+            return self.astng_from_module_name(modname)
+        raise ASTNGBuildingException('unable to get astng for file %s' %
+                                     filepath)
 
     def astng_from_module_name(self, modname, context_file=None):
         """given a module name, return the astng object"""
+        if modname in self._cache:
+            return self._cache[modname]
         old_cwd = os.getcwd()
         if context_file:
             os.chdir(dirname(context_file))
         try:
             filepath = self.file_from_module_name(modname, context_file)
             if filepath is not None and not is_python_source(filepath):
-                try:
-                    return self._cache[filepath]
-                except KeyError:
-                    data, zmodname = zip_import_data(filepath)
-                    if data is not None:
-                        from logilab.astng.builder import ASTNGBuilder
-                        try:
-                            astng = ASTNGBuilder(self).string_build(data, zmodname, filepath)
-                        except (SyntaxError, KeyboardInterrupt, SystemExit):
-                            raise
-                        self._cache[filepath] = astng
-                        return astng
+                data, zmodname = zip_import_data(filepath)
+                if data is not None:
+                    from logilab.astng.builder import ASTNGBuilder
+                    try:
+                        return ASTNGBuilder(self).string_build(data, zmodname,
+                                                               filepath)
+                    except (SyntaxError, KeyboardInterrupt, SystemExit):
+                        raise
             if filepath is None or not is_python_source(filepath):
                 try:
                     module = load_module_from_name(modname)
-                except ImportError, ex:
+                # catch SystemError as well, we may get that on badly
+                # initialized C-module
+                except (SystemError, ImportError), ex:
                     msg = 'Unable to load module %s (%s)' % (modname, ex)
                     raise ASTNGBuildingException(msg)
                 return self.astng_from_module(module, modname)
@@ -192,7 +202,8 @@ class ASTNGManager(OptionsProviderMixIn):
     def astng_from_module(self, module, modname=None):
         """given an imported module, return the astng object"""
         modname = modname or module.__name__
-        filepath = modname
+        if modname in self._cache:
+            return self._cache[modname]
         try:
             # some builtin modules don't have __file__ attribute
             filepath = module.__file__
@@ -200,15 +211,8 @@ class ASTNGManager(OptionsProviderMixIn):
                 return self.astng_from_file(filepath, modname)
         except AttributeError:
             pass
-        try:
-            return self._cache[filepath]
-        except KeyError:
-            from logilab.astng.builder import ASTNGBuilder
-            astng = ASTNGBuilder(self).module_build(module, modname)
-            # update caches (filepath and astng.file are not necessarily  the
-            # same (.pyc pb))
-            self._cache[filepath] = self._cache[astng.file] = astng
-            return astng
+        from logilab.astng.builder import ASTNGBuilder
+        return ASTNGBuilder(self).module_build(module, modname)
 
     def astng_from_class(self, klass, modname=None):
         """get astng for the given class"""
@@ -236,7 +240,7 @@ class ASTNGManager(OptionsProviderMixIn):
                     'Unable to get module for %s' % safe_repr(klass))
             except Exception, ex:
                 raise ASTNGBuildingException(
-                    'Unexpected error while retreiving module for %s: %s'
+                    'Unexpected error while retrieving module for %s: %s'
                     % (safe_repr(klass), ex))
         try:
             name = klass.__name__
@@ -245,14 +249,16 @@ class ASTNGManager(OptionsProviderMixIn):
                 'Unable to get name for %s' % safe_repr(klass))
         except Exception, ex:
             raise ASTNGBuildingException(
-                'Unexpected error while retreiving name for %s: %s'
+                'Unexpected error while retrieving name for %s: %s'
                 % (safe_repr(klass), ex))
         # take care, on living object __module__ is regularly wrong :(
         modastng = self.astng_from_module_name(modname)
-        for infered in modastng.igetattr(name, context):
-            if klass is not obj and isinstance(infered, nodes.Class):
-                infered = infutils.Instance(infered)
-            yield infered
+        if klass is obj:
+            for  infered in modastng.igetattr(name, context):
+                yield infered
+        else:
+            for infered in modastng.igetattr(name, context):
+                yield infered.instanciate_class()
 
     def project_from_files(self, files, func_wrapper=astng_wrapper,
                            project_name=None, black_list=None):
@@ -412,3 +418,4 @@ class Project:
     def __repr__(self):
         return '<Project %r at %s (%s modules)>' % (self.name, id(self),
                                                     len(self.modules))
+
