@@ -1,34 +1,49 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 """this module contains a set of functions to handle python protocols for nodes
 where it makes sense.
 
-:author:    Sylvain Thenault
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2009 Sylvain Thenault
-:contact:   mailto:thenault@gmail.com
 """
 
 from __future__ import generators
 
 __doctype__ = "restructuredtext en"
 
-from logilab.astng import InferenceError, NoDefault, _nodes as nodes
-from logilab.astng.infutils import copy_context, unpack_infer, \
+from logilab.astng import InferenceError, NoDefault
+from logilab.astng.node_classes import unpack_infer
+from logilab.astng.bases import copy_context, \
      raise_if_nothing_infered, yes_if_nothing_infered, Instance, Generator, YES
-from logilab.astng._nodes import Const, Class, Function, Tuple, List, \
-     const_factory
+from logilab.astng.nodes import const_factory
+from logilab.astng import nodes
 
 # unary operations ############################################################
 
@@ -50,11 +65,11 @@ nodes.Dict.infer_unary_op = dict_infer_unary_op
 def const_infer_unary_op(self, operator):
     if operator == 'not':
         return const_factory(not self.value)
-    # XXX log potentialy raised TypeError
+    # XXX log potentially raised TypeError
     elif operator == '+':
         return const_factory(+self.value)
     else: # operator == '-':
-        return const_factory(-self.value)        
+        return const_factory(-self.value)
 nodes.Const.infer_unary_op = const_infer_unary_op
 
 
@@ -73,13 +88,21 @@ BIN_OP_IMPL = {'+':  lambda a, b: a + b,
                '<<': lambda a, b: a << b,
                '>>': lambda a, b: a >> b,
                }
+for key, impl in BIN_OP_IMPL.items():
+    BIN_OP_IMPL[key+'='] = impl
 
 def const_infer_binary_op(self, operator, other, context):
     for other in other.infer(context):
-        if isinstance(other, Const):
+        if isinstance(other, nodes.Const):
             try:
                 impl = BIN_OP_IMPL[operator]
-                yield const_factory(impl(self.value, other.value))
+
+                try:
+                    yield const_factory(impl(self.value, other.value))
+                except StandardError:
+                    # ArithmeticError is not enough: float >> float is a TypeError
+                    # TODO : let pylint know about the problem
+                    pass
             except TypeError:
                 # XXX log TypeError
                 continue
@@ -93,21 +116,24 @@ def const_infer_binary_op(self, operator, other, context):
                 yield YES
 nodes.Const.infer_binary_op = yes_if_nothing_infered(const_infer_binary_op)
 
-    
+
 def tl_infer_binary_op(self, operator, other, context):
     for other in other.infer(context):
         if isinstance(other, self.__class__) and operator == '+':
             node = self.__class__()
-            elts = [n for elt in self.elts for n in elt.infer(context)]
-            elts += [n for elt in other.elts for n in elt.infer(context)]
+            elts = [n for elt in self.elts for n in elt.infer(context)
+                    if not n is YES]
+            elts += [n for elt in other.elts for n in elt.infer(context)
+                     if not n is YES]
             node.elts = elts
             yield node
-        elif isinstance(other, Const) and operator == '*':
+        elif isinstance(other, nodes.Const) and operator == '*':
             node = self.__class__()
-            elts = [n for elt in self.elts for n in elt.infer(context)] * other.value
+            elts = [n for elt in self.elts for n in elt.infer(context)
+                    if not n is YES] * other.value
             node.elts = elts
             yield node
-        elif isinstance(other, Instance) and not isinstance(other, Const):
+        elif isinstance(other, Instance) and not isinstance(other, nodes.Const):
             yield YES
     # XXX else log TypeError
 nodes.Tuple.infer_binary_op = yes_if_nothing_infered(tl_infer_binary_op)
@@ -116,7 +142,7 @@ nodes.List.infer_binary_op = yes_if_nothing_infered(tl_infer_binary_op)
 
 def dict_infer_binary_op(self, operator, other, context):
     for other in other.infer(context):
-        if isinstance(other, Instance) and isinstance(other._proxied, Class):
+        if isinstance(other, Instance) and isinstance(other._proxied, nodes.Class):
             yield YES
         # XXX else log TypeError
 nodes.Dict.infer_binary_op = yes_if_nothing_infered(dict_infer_binary_op)
@@ -125,7 +151,7 @@ nodes.Dict.infer_binary_op = yes_if_nothing_infered(dict_infer_binary_op)
 # assignment ##################################################################
 
 """the assigned_stmts method is responsible to return the assigned statement
-(eg not infered) according to the assignment type.
+(e.g. not inferred) according to the assignment type.
 
 The `asspath` argument is used to record the lhs path of the original node.
 For instance if we want assigned statements for 'c' in 'a, (b,c)', asspath
@@ -154,17 +180,20 @@ def _resolve_looppart(parts, asspath, context):
                 assigned = stmt.getitem(index, context)
             except (AttributeError, IndexError):
                 continue
+            except TypeError, exc: # stmt is unsubscriptable Const
+                continue
             if not asspath:
-                # we acheived to resolved the assigment path,
+                # we achieved to resolved the assignment path,
                 # don't infer the last part
                 yield assigned
             elif assigned is YES:
                 break
             else:
                 # we are not yet on the last part of the path
-                # search on each possibly infered value
+                # search on each possibly inferred value
                 try:
-                    for infered in _resolve_looppart(assigned.infer(context), asspath, context):
+                    for infered in _resolve_looppart(assigned.infer(context),
+                                                     asspath, context):
                         yield infered
                 except InferenceError:
                     break
@@ -173,12 +202,14 @@ def _resolve_looppart(parts, asspath, context):
 def for_assigned_stmts(self, node, context=None, asspath=None):
     if asspath is None:
         for lst in self.iter.infer(context):
-            if isinstance(lst, (Tuple, List)):
+            if isinstance(lst, (nodes.Tuple, nodes.List)):
                 for item in lst.elts:
                     yield item
     else:
-        for infered in _resolve_looppart(self.iter.infer(context), asspath, context):
+        for infered in _resolve_looppart(self.iter.infer(context),
+                                             asspath, context):
             yield infered
+
 nodes.For.assigned_stmts = raise_if_nothing_infered(for_assigned_stmts)
 nodes.Comprehension.assigned_stmts = raise_if_nothing_infered(for_assigned_stmts)
 
@@ -193,13 +224,13 @@ nodes.List.assigned_stmts = mulass_assigned_stmts
 
 
 def assend_assigned_stmts(self, context=None):
-    return self.parent.assigned_stmts(self, context=context)    
+    return self.parent.assigned_stmts(self, context=context)
 nodes.AssName.assigned_stmts = assend_assigned_stmts
 nodes.AssAttr.assigned_stmts = assend_assigned_stmts
 
 
 def _arguments_infer_argname(self, name, context):
-    # arguments informmtion may be missing, in which case we can't do anything
+    # arguments information may be missing, in which case we can't do anything
     # more
     if not (self.args or self.vararg or self.kwarg):
         yield YES
@@ -242,7 +273,7 @@ def arguments_assigned_stmts(self, node, context, asspath=None):
     for infered in _arguments_infer_argname(self, node.name, context):
         yield infered
 nodes.Arguments.assigned_stmts = arguments_assigned_stmts
-        
+
 
 def assign_assigned_stmts(self, node, context=None, asspath=None):
     if not asspath:
@@ -267,25 +298,25 @@ def _resolve_asspart(parts, asspath, context):
             except (TypeError, IndexError):
                 return
             if not asspath:
-                # we acheived to resolved the assigment path, don't infer the
+                # we achieved to resolved the assignment path, don't infer the
                 # last part
                 yield assigned
             elif assigned is YES:
                 return
             else:
                 # we are not yet on the last part of the path search on each
-                # possibly infered value
+                # possibly inferred value
                 try:
-                    for infered in _resolve_asspart(assigned.infer(context), 
+                    for infered in _resolve_asspart(assigned.infer(context),
                                                     asspath, context):
                         yield infered
                 except InferenceError:
                     return
 
-    
+
 def excepthandler_assigned_stmts(self, node, context=None, asspath=None):
     for assigned in unpack_infer(self.type):
-        if isinstance(assigned, Class):
+        if isinstance(assigned, nodes.Class):
             assigned = Instance(assigned)
         yield assigned
 nodes.ExceptHandler.assigned_stmts = raise_if_nothing_infered(excepthandler_assigned_stmts)
@@ -294,41 +325,9 @@ nodes.ExceptHandler.assigned_stmts = raise_if_nothing_infered(excepthandler_assi
 def with_assigned_stmts(self, node, context=None, asspath=None):
     if asspath is None:
         for lst in self.vars.infer(context):
-            if isinstance(lst, (Tuple, List)):
+            if isinstance(lst, (nodes.Tuple, nodes.List)):
                 for item in lst.nodes:
                     yield item
 nodes.With.assigned_stmts = raise_if_nothing_infered(with_assigned_stmts)
 
 
-def parent_ass_type(self, context=None):
-    return self.parent.ass_type()
-    
-nodes.Tuple.ass_type = parent_ass_type
-nodes.List.ass_type = parent_ass_type
-nodes.AssName.ass_type = parent_ass_type
-nodes.AssAttr.ass_type = parent_ass_type
-nodes.DelName.ass_type = parent_ass_type
-nodes.DelAttr.ass_type = parent_ass_type
-
-def end_ass_type(self):
-    return self
-
-# XXX if you add ass_type to a class, you should probably modify
-#     lookup.LookupMixIn.filter_stmts around line ::
-#
-#       if ass_type is mystmt and not isinstance(ass_type, (nodes.Class, ...)):
-nodes.Arguments.ass_type = end_ass_type
-nodes.Assign.ass_type = end_ass_type
-nodes.AugAssign.ass_type = end_ass_type
-nodes.Class.ass_type = end_ass_type
-nodes.Comprehension.ass_type = end_ass_type
-nodes.Delete.ass_type = end_ass_type
-nodes.ExceptHandler.ass_type = end_ass_type
-nodes.For.ass_type = end_ass_type
-nodes.From.ass_type = end_ass_type
-nodes.Function.ass_type = end_ass_type
-nodes.Import.ass_type = end_ass_type
-nodes.With.ass_type = end_ass_type
-
-
-
