@@ -1,28 +1,38 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
 #
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
-"""This module extends ast "scoped" node, i.e. which are opening a new
-local scope in the language definition : Module, Class, Function (and
-Lambda to some extends).
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
+"""This module contains the classes for "scoped" node, i.e. which are opening a
+new local scope in the language definition : Module, Class, Function (and
+Lambda and GenExpr to some extends).
 
-All new methods and attributes added on each class are documented
-below.
-
-
-:author:    Sylvain Thenault
-:copyright: 2003-2009 LOGILAB S.A. (Paris, FRANCE)
-:contact:   http://www.logilab.fr/ -- mailto:python-projects@logilab.org
-:copyright: 2003-2009 Sylvain Thenault
-:contact:   mailto:thenault@gmail.com
 """
 from __future__ import generators
 
@@ -36,14 +46,15 @@ from logilab.common.decorators import cached
 
 from logilab.astng import MANAGER, NotFoundError, NoDefault, \
      ASTNGBuildingException, InferenceError
-from logilab.astng._nodes import (Arguments, Class, Const, Dict, From, Function,
-     GenExpr, Lambda, List, Module, Name, Pass, Raise, Return, Tuple, Yield,
-     AssName, DelAttr, DelName, const_factory as cf, NodeNG, StmtMixIn)
+from logilab.astng.node_classes import (Const, DelName, DelAttr,
+     Dict, From, List, Name, Pass, Raise, Return, Tuple, Yield,
+     are_exclusive, LookupMixIn, const_factory as cf, unpack_infer)
+from logilab.astng.bases import (NodeNG, BaseClass, InferenceContext, Instance,
+     YES, Generator, UnboundMethod, BoundMethod, _infer_stmts, copy_context)
+from logilab.astng.mixins import (StmtMixIn, FilterStmtsMixin)
 
-from logilab.astng.infutils import YES, InferenceContext, Instance, Generator, \
-     UnboundMethod, BoundMethod, copy_context, unpack_infer, _infer_stmts
 from logilab.astng.nodes_as_string import as_string
-from logilab.astng.lookup import LookupMixIn, LocalsDictMixIn
+
 
 def remove_nodes(func, cls):
     def wrapper(*args, **kwargs):
@@ -76,14 +87,148 @@ def std_special_attributes(self, name, add_locals=True):
     raise NotFoundError(name)
 
 
+
+def builtin_lookup(name):
+    """lookup a name into the builtin module
+    return the list of matching statements and the astng for the builtin
+    module
+    """
+    # TODO : once there is no more monkey patching, make a BUILTINASTNG const
+    builtinastng = MANAGER.astng_from_module(__builtin__)
+    if name == '__dict__':
+        return builtinastng, ()
+    try:
+        stmts = builtinastng.locals[name]
+    except KeyError:
+        stmts = ()
+    return builtinastng, stmts
+
+
+# TODO move this Mixin to mixins.py; problem: 'Function' in _scope_lookup
+class LocalsDictNodeNG(LookupMixIn, NodeNG):
+    """ this class provides locals handling common to Module, Function
+    and Class nodes, including a dict like interface for direct access
+    to locals information
+    """
+
+    # attributes below are set by the builder module or by raw factories
+
+    # dictionary of locals with name as key and node defining the local as
+    # value
+
+    def qname(self):
+        """return the 'qualified' name of the node, eg module.name,
+        module.class.name ...
+        """
+        if self.parent is None:
+            return self.name
+        return '%s.%s' % (self.parent.frame().qname(), self.name)
+
+    def frame(self):
+        """return the first parent frame node (i.e. Module, Function or Class)
+        """
+        return self
+
+    def scope(self):
+        """return the first node defining a new scope (i.e. Module,
+        Function, Class, Lambda but also GenExpr)
+        """
+        return self
+
+
+    def _scope_lookup(self, node, name, offset=0):
+        """XXX method for interfacing the scope lookup"""
+        try:
+            stmts = node._filter_stmts(self.locals[name], self, offset)
+        except KeyError:
+            stmts = ()
+        if stmts:
+            return self, stmts
+        if self.parent: # i.e. not Module
+            # nested scope: if parent scope is a function, that's fine
+            # else jump to the module
+            pscope = self.parent.scope()
+            if not isinstance(pscope, Function):
+                pscope = pscope.root()
+            return pscope.scope_lookup(node, name)
+        return builtin_lookup(name) # Module
+
+
+
+    def set_local(self, name, stmt):
+        """define <name> in locals (<stmt> is the node defining the name)
+        if the node is a Module node (i.e. has globals), add the name to
+        globals
+
+        if the name is already defined, ignore it
+        """
+        #assert not stmt in self.locals.get(name, ()), (self, stmt)
+        self.locals.setdefault(name, []).append(stmt)
+
+    __setitem__ = set_local
+
+    def _append_node(self, child):
+        """append a child, linking it in the tree"""
+        self.body.append(child)
+        child.parent = self
+
+    def add_local_node(self, child_node, name=None):
+        """append a child which should alter locals to the given node"""
+        if name != '__class__':
+            # add __class__ node as a child will cause infinite recursion later!
+            self._append_node(child_node)
+        self.set_local(name or child_node.name, child_node)
+
+
+    def __getitem__(self, item):
+        """method from the `dict` interface returning the first node
+        associated with the given name in the locals dictionary
+
+        :type item: str
+        :param item: the name of the locally defined object
+        :raises KeyError: if the name is not defined
+        """
+        return self.locals[item][0]
+
+    def __iter__(self):
+        """method from the `dict` interface returning an iterator on
+        `self.keys()`
+        """
+        return iter(self.keys())
+
+    def keys(self):
+        """method from the `dict` interface returning a tuple containing
+        locally defined names
+        """
+        return self.locals.keys()
+
+    def values(self):
+        """method from the `dict` interface returning a tuple containing
+        locally defined nodes which are instance of `Function` or `Class`
+        """
+        return [self[key] for key in self.keys()]
+
+    def items(self):
+        """method from the `dict` interface returning a list of tuple
+        containing each locally defined name with its associated node,
+        which is an instance of `Function` or `Class`
+        """
+        return zip(self.keys(), self.values())
+
+    def has_key(self, name):
+        """method from the `dict` interface returning True if the given
+        name is defined in the locals dictionary
+        """
+        return self.locals.has_key(name)
+
+    __contains__ = has_key
+
+
 # Module  #####################################################################
 
-class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
-    """/!\ this class should not be used directly /!\ it's
-    only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionary
-    (see below the class definition)
-    """
+class Module(LocalsDictNodeNG):
+    _astng_fields = ('body',)
+
     fromlineno = 0
     lineno = 0
 
@@ -108,6 +253,13 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
     # names of module attributes available through the global scope
     scope_attrs = set(('__name__', '__doc__', '__file__', '__path__'))
 
+    def __init__(self, name, doc, pure_python=True):
+        self.name = name
+        self.doc = doc
+        self.pure_python = pure_python
+        self.locals = self.globals = {}
+        self.body = []
+
     # Module is not a Statement node but needs the replace method (see StmtMixIn)
     def replace(self, child, newchild):
         sequence = self.child_sequence(child)
@@ -118,7 +270,7 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
     def block_range(self, lineno):
         """return block line numbers.
 
-        start from the "class" position whatever the given lineno
+        start from the beginning whatever the given lineno
         """
         return self.fromlineno, self.tolineno
 
@@ -160,7 +312,7 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
     getattr = remove_nodes(getattr, DelName)
 
     def igetattr(self, name, context=None):
-        """infered getattr"""
+        """inferred getattr"""
         # set lookup name since this is necessary to infer on import nodes for
         # instance
         context = copy_context(context)
@@ -226,7 +378,7 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
         return modname
 
     def wildcard_import_names(self):
-        """return the list of imported names when this module is 'wildard
+        """return the list of imported names when this module is 'wildcard
         imported'
 
         It doesn't include the '__builtins__' name which is added by the
@@ -256,6 +408,10 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
             explicit = all.assigned_stmts().next()
         except InferenceError:
             return [name for name in self.keys() if not name.startswith('_')]
+        except AttributeError:
+            # not an assignment node
+            # XXX infer?
+            return [name for name in self.keys() if not name.startswith('_')]
         try:
             # should be a Tuple/List of constant string / 1 string not allowed
             return [const.value for const in explicit.elts]
@@ -263,25 +419,33 @@ class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
             return [name for name in self.keys() if not name.startswith('_')]
 
 
-class GenExprNG(LookupMixIn, LocalsDictMixIn, NodeNG):
-    """class representing a GenExpr node"""
+class GenExpr(LocalsDictNodeNG):
+    _astng_fields = ('elt', 'generators')
+
+    def __init__(self):
+        self.locals = {}
+        self.elt = None
+        self.generators = []
 
     def frame(self):
         return self.parent.frame()
+GenExpr.scope_lookup = LocalsDictNodeNG._scope_lookup
 
 
 # Function  ###################################################################
 
-class LambdaNG(LookupMixIn, LocalsDictMixIn, NodeNG):
-    """/!\ this class should not be used directly /!\ it's
-    only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionary
-    (see below the class definition)
-    """
+
+class Lambda(LocalsDictNodeNG, FilterStmtsMixin):
+    _astng_fields = ('args', 'body',)
 
     # function's type, 'function' | 'method' | 'staticmethod' | 'classmethod'
     type = 'function'
 
+    def __init__(self):
+        self.locals = {}
+        self.args = []
+        self.body = []
+
     def pytype(self):
         if 'method' in self.type:
             return '__builtin__.instancemethod'
@@ -322,20 +486,21 @@ class LambdaNG(LookupMixIn, LocalsDictMixIn, NodeNG):
             frame = self
         return frame._scope_lookup(node, name, offset)
 
-
-class FunctionNG(LambdaNG, LookupMixIn, LocalsDictMixIn, StmtMixIn, NodeNG):
-    """/!\ this class should not be used directly /!\ it's
-    only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionary
-    (see below the class definition)
-    """
-
+class Function(StmtMixIn, Lambda):
+    _astng_fields = ('decorators', 'args', 'body')
 
     special_attributes = set(('__name__', '__doc__', '__dict__'))
     # attributes below are set by the builder module or by raw factories
 
     blockstart_tolineno = None
 
+    def __init__(self, name, doc):
+        self.locals = {}
+        self.args = []
+        self.body = []
+        self.decorators = None
+        self.name = name
+        self.doc = doc
 
     def set_line_info(self, lastchild):
         self.fromlineno = self.lineno
@@ -348,7 +513,7 @@ class FunctionNG(LambdaNG, LookupMixIn, LocalsDictMixIn, StmtMixIn, NodeNG):
     def block_range(self, lineno):
         """return block line numbers.
 
-        start from the "class" position whatever the given lineno
+        start from the "def" position whatever the given lineno
         """
         return self.fromlineno, self.tolineno
 
@@ -363,7 +528,7 @@ class FunctionNG(LambdaNG, LookupMixIn, LocalsDictMixIn, StmtMixIn, NodeNG):
     def is_method(self):
         """return true if the function node should be considered as a method"""
         # check we are defined in a Class, because this is usually expected
-        # (eg pylint...) when is_method() return True
+        # (e.g. pylint...) when is_method() return True
         return self.type != 'function' and isinstance(self.parent.frame(), Class)
 
     @cached
@@ -488,12 +653,15 @@ def _iface_hdlr(iface_node):
     return True
 
 
-class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
-    """/!\ this class should not be used directly /!\ it's
-    only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionary
-    (see below the class definition)
-    """
+class Class(StmtMixIn, LocalsDictNodeNG, FilterStmtsMixin):
+
+    # some of the attributes below are set by the builder module or
+    # by a raw factories
+
+    # a dictionary of class instances attributes
+    _astng_fields = ('bases', 'body',) # name
+
+    instance_attrs = None
     special_attributes = set(('__name__', '__doc__', '__dict__', '__module__',
                               '__bases__', '__mro__'))
 
@@ -504,6 +672,14 @@ class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
                     doc="class'type, possible values are 'class' | "
                     "'metaclass' | 'interface' | 'exception'")
 
+    def __init__(self, name, doc):
+        self.instance_attrs = {}
+        self.locals = {}
+        self.bases = []
+        self.body = []
+        self.name = name
+        self.doc = doc
+
     def _newstyle_impl(self, context=None):
         if context is None:
             context = InferenceContext()
@@ -561,12 +737,7 @@ class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
             frame = self
         return frame._scope_lookup(node, name, offset)
 
-    # attributes below are set by the builder module or by raw factories
-
-    # a dictionary of class instances attributes
-    instance_attrs = None
-
-    # list of parent class as a list of string (ie names as they appears
+    # list of parent class as a list of string (i.e. names as they appear
     # in the class definition) XXX bw compat
     def basenames(self):
         return [as_string(bnode) for bnode in self.bases]
@@ -619,6 +790,9 @@ class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
             if astng.instance_attrs.has_key(name):
                 yield astng
 
+    def has_base(self, node):
+        return node in self.bases
+
     def local_attr(self, name, context=None):
         """return the list of assign node associated to name in this class
         locals or in its parents
@@ -653,6 +827,10 @@ class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
         return values
     instance_attr = remove_nodes(instance_attr, DelAttr)
 
+    def instanciate_class(self):
+        """return Instance of Class node, else return self"""
+        return Instance(self)
+
     def getattr(self, name, context=None):
         """this method doesn't look in the instance_attrs dictionary since it's
         done by an Instance proxy at inference time.
@@ -683,10 +861,10 @@ class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
         return values
 
     def igetattr(self, name, context=None):
-        """infered getattr, need special treatment in class to handle
+        """inferred getattr, need special treatment in class to handle
         descriptors
         """
-        # set lookoup name since this is necessary to infer on import nodes for
+        # set lookup name since this is necessary to infer on import nodes for
         # instance
         context = copy_context(context)
         context.lookupname = name
