@@ -1,13 +1,32 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """tests for the astng builder and rebuilder module"""
@@ -19,9 +38,10 @@ from os.path import join, abspath, dirname
 from logilab.common.testlib import TestCase, unittest_main
 from pprint import pprint
 
-from logilab.astng import builder, nodes, patchcomptransformer, MANAGER, InferenceError
+from logilab.astng import builder, nodes, patchcomptransformer, MANAGER, \
+                          InferenceError, NotFoundError
 from logilab.astng.nodes import Module
-from logilab.astng.infutils import YES
+from logilab.astng.bases import YES
 from logilab.astng.nodes_as_string import as_string
 
 from unittest_inference import get_name_node
@@ -406,7 +426,7 @@ def global_no_effect():
         self.assertIsInstance(astng.getattr('CSTE')[0], nodes.AssName)
         self.failUnlessEqual(astng.getattr('CSTE')[0].fromlineno, 2)
         self.failUnlessEqual(astng.getattr('CSTE')[1].fromlineno, 6)
-        self.assertRaises(nodes.NotFoundError,
+        self.assertRaises(NotFoundError,
                           astng.getattr, 'CSTE2')
         self.assertRaises(InferenceError,
                           astng['global_no_effect'].ilookup('CSTE2').next)
@@ -414,7 +434,7 @@ def global_no_effect():
     def test_socket_build(self):
         import socket
         astng = self.builder.module_build(socket)
-        # XXX just check the first one. Actually 3 objects are infered (look at
+        # XXX just check the first one. Actually 3 objects are inferred (look at
         # the socket module) but the last one as those attributes dynamically
         # set and astng is missing this.
         for fclass in astng.igetattr('socket'):
@@ -428,7 +448,7 @@ def global_no_effect():
         def test_gen_expr_var_scope(self):
             data = 'l = list(n for n in range(10))\n'
             astng = self.builder.string_build(data, __name__, __file__)
-            # n unvailable touside gen expr scope
+            # n unavailable outside gen expr scope
             self.failIf('n' in astng)
             # test n is inferable anyway
             n = get_name_node(astng, 'n')
@@ -467,8 +487,8 @@ class FileBuildTC(TestCase):
         keys.sort()
         should = ['MY_DICT', 'YO', 'YOUPI',
                 '__revision__',  'global_access','modutils', 'nested_args',
-                 'os', 'redirect', 'spawn', 'REDIRECT', 'LocalsVisitor',
-                'ASTWalker', 'ASTVisitor',  'extend_class']
+                 'os', 'redirect', 'spawn', 'LocalsVisitor',
+                'ASTWalker', 'ASTVisitor']
         should.sort()
         self.assertEquals(keys, should)
 
@@ -520,7 +540,8 @@ class FileBuildTC(TestCase):
         locals2 = klass2.locals
         keys = locals2.keys()
         keys.sort()
-        self.assertEquals(keys, ['__init__', 'class_attr', 'class_method', 'method', 'static_method'])
+        self.assertEquals(keys, ['__init__', 'class_attr', 'class_method',
+                                 'method', 'static_method'])
 
     def test_class_instance_attrs(self):
         module = self.module
@@ -598,7 +619,7 @@ A.ass_type = A_ass_type
 #         import compiler
 #         sn = astng.MANAGER.astng_from_file(join(astng.__path__[0], 'inference.py'))
 #         astastng = astng.MANAGER.astng_from_file(join(compiler.__path__[0], 'ast.py'))
-#         # check monkey patching of the compiler module has been infered
+#         # check monkey patching of the compiler module has been inferred
 #         lclass = list(astastng.igetattr('Function'))
 #         self.assertEquals(len(lclass), 1)
 #         lclass = lclass[0]
@@ -642,5 +663,22 @@ def visit_if(self, node):
         astng = self.builder.string_build(code)
         self.failUnless('body' in astng['visit_if'].locals)
 
+    def test_build_constants(self):
+        '''test expected values of constants after rebuilding'''
+        code = '''
+def func():
+    return None
+    return
+    return 'None'
+'''
+        astng = self.builder.string_build(code)
+        none, nothing, chain = [ret.value for ret in astng.body[0].body]
+        self.assertIsInstance(none, nodes.Const)
+        self.assertEquals(none.value, None)
+        self.assertEquals(nothing, None)
+        self.assertIsInstance(chain, nodes.Const)
+        self.assertEquals(chain.value, 'None')
+
+
 if __name__ == '__main__':
     unittest_main()
