@@ -1,13 +1,32 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """tests for the astng inference capabilities
@@ -18,7 +37,7 @@ from StringIO import StringIO
 from logilab.common.testlib import TestCase, unittest_main
 
 from logilab.astng import InferenceError, builder, nodes, inference
-from logilab.astng.infutils import YES, Instance, BoundMethod, UnboundMethod, path_wrapper
+from logilab.astng.bases import YES, Instance, BoundMethod, UnboundMethod, path_wrapper
 
 def get_name_node(start_from, name, index=0):
     return [n for n in start_from.nodes_of_class(nodes.Name) if n.name == name][index]
@@ -31,7 +50,9 @@ builder = builder.ASTNGBuilder()
 class InferenceUtilsTC(TestCase):
 
     def test_path_wrapper(self):
-        infer_default = path_wrapper(inference.infer_default)
+        def infer_default(self, *args):
+            raise InferenceError
+        infer_default = path_wrapper(infer_default)
         infer_end = path_wrapper(inference.infer_end)
         self.failUnlessRaises(InferenceError,
                               infer_default(1).next)
@@ -551,8 +572,8 @@ if __name__ == '__main__':
         self.failUnlessEqual(len(list(astng['process_line'].infer_call_result(
                                                                 None))), 3)
         self.failUnlessEqual(len(list(astng['tupletest'].infer())), 3)
-        values = ['Function(first_word)', 'Function(last_word)', 'Const(None)']
-        self.failUnlessEqual([str(infered).replace('FunctionDef', 'Function')
+        values = ['Function(first_word)', 'Function(last_word)', 'Const(NoneType)']
+        self.failUnlessEqual([str(infered)
                               for infered in astng['fct'].infer()], values)
 
     def test_float_complex_ambiguity(self):
@@ -656,12 +677,12 @@ print ((d,e) for e,d in ([1,2], [3,4]))
         code = '''
 help()
         '''
-        # XXX failing with python > 2.3 since __builtin__.help assigment has
+        # XXX failing with python > 2.3 since __builtin__.help assignment has
         #     been moved into a function...
         astng = builder.string_build(code, __name__, __file__)
         node = get_name_node(astng, 'help', -1)
         infered = list(node.infer())
-        self.failUnlessEqual(len(infered), 1)
+        self.failUnlessEqual(len(infered), 1, infered)
         self.assertIsInstance(infered[0], Instance)
         self.failUnlessEqual(str(infered[0]),
                              'Instance of site._Helper')
@@ -1004,6 +1025,7 @@ print a
 '''
         astng = builder.string_build(code, __name__, __file__)
         infered = list(get_name_node(astng, 'a').infer())
+
         self.assertEquals(len(infered), 1)
         self.assertIsInstance(infered[0], nodes.Const)
         self.assertEquals(infered[0].value, 3)
@@ -1053,5 +1075,38 @@ class DataManager(object):
                 break
         else:
             self.fail('expected to find an instance of Application in %s' % infered)
+
+    def test_list_inference(self):
+        """#20464"""
+        code = '''
+import optparse
+
+A = []
+B = []
+
+def test():
+  xyz = [
+    "foobar=%s" % options.ca,
+  ] + A + B
+
+  if options.bind is not None:
+    xyz.append("bind=%s" % options.bind)
+  return xyz
+
+def main():
+  global options
+
+  parser = optparse.OptionParser()
+  (options, args) = parser.parse_args()
+
+Z = test()
+        '''
+        astng = builder.string_build(code, __name__, __file__)
+        infered = list(astng['Z'].infer())
+        self.assertEquals(len(infered), 1, infered)
+        self.assertIsInstance(infered[0], Instance)
+        self.assertIsInstance(infered[0]._proxied, nodes.Class)
+        self.assertEquals(infered[0]._proxied.name, 'list')
+
 if __name__ == '__main__':
     unittest_main()
