@@ -1,13 +1,32 @@
 # This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
+# the terms of the GNU Lesser General Public License as published by the Free Software
 # Foundation; either version 2 of the License, or (at your option) any later
 # version.
+# copyright 2003-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
+# copyright 2003-2010 Sylvain Thenault, all rights reserved.
+# contact mailto:thenault@gmail.com
+#
+# This file is part of logilab-astng.
+#
+# logilab-astng is free software: you can redistribute it and/or modify it
+# under the terms of the GNU Lesser General Public License as published by the
+# Free Software Foundation, either version 2.1 of the License, or (at your
+# option) any later version.
+#
+# logilab-astng is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
+# for more details.
+#
+# You should have received a copy of the GNU Lesser General Public License along
+# with logilab-astng. If not, see <http://www.gnu.org/licenses/>.
 
 # This program is distributed in the hope that it will be useful, but WITHOUT
 # ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License along with
+# You should have received a copy of the GNU Lesser General Public License along with
 # this program; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 """tests for the astng variable lookup capabilities
@@ -16,9 +35,9 @@ import sys
 from os.path import join, abspath
 from logilab.common.testlib import TestCase, unittest_main
 
-from logilab.astng import builder, nodes, scoped_nodes, lookup, \
+from logilab.astng import builder, nodes, scoped_nodes, \
      InferenceError, NotFoundError
-
+from logilab.astng.scoped_nodes import builtin_lookup
 from unittest_inference import get_name_node
 
 builder = builder.ASTNGBuilder()
@@ -127,9 +146,9 @@ if x > 0:
     print '#' * x        
         """, __name__, __file__)
         xnames = [n for n in astng.nodes_of_class(nodes.Name) if n.name == 'x']
-        # inside the loop, only one possible assigment
+        # inside the loop, only one possible assignment
         self.assertEquals(len(xnames[0].lookup('x')[1]), 1)
-        # outside the loop, two possible assigments
+        # outside the loop, two possible assignments
         self.assertEquals(len(xnames[1].lookup('x')[1]), 2)
         self.assertEquals(len(xnames[2].lookup('x')[1]), 2)
 
@@ -183,8 +202,8 @@ def initialize(linter):
 
 
     def test_builtin_lookup(self):
-        self.assertEquals(lookup.builtin_lookup('__dict__')[1], ())
-        intstmts = lookup.builtin_lookup('int')[1]
+        self.assertEquals(builtin_lookup('__dict__')[1], ())
+        intstmts = builtin_lookup('int')[1]
         self.assertEquals(len(intstmts), 1)
         self.assertIsInstance(intstmts[0], nodes.Class)
         self.assertEquals(intstmts[0].name, 'int')
